.getExtendsClass <-
    function(rType)
{
    classDef <- getClassDef(rType)
    if (is.null(classDef))
        character(0)
    else {
        contains <- classDef@contains
        keep <- sapply(contains, slot, "distance") == 1
        if (sum(keep) > 1)
            stop("multiple inheritance in class '", rClass, "'")
        if (length(keep) == 0) character(0)
        else names(contains)[[which(keep)]]
    }
}

setGeneric(".getThisClassSlots",
           function(typeMode, rExtends, allSlots)
           standardGeneric(".getThisClassSlots"),
           signature="typeMode")

setMethod(.getThisClassSlots, "typeMode",
    function(typeMode, rExtends, allSlots) allSlots)

setMethod(.getThisClassSlots, "hierarchy",
    function(typeMode, rExtends, allSlots)
{
    if (length(rExtends) == 0) allSlots
    else {
        idx <- !names(allSlots) %in% slotNames(rExtends)
        allSlots[idx]
    }
})

setGeneric(".getExtendsClassSlots",
           function(typeMode, rExtends, allSlots)
           standardGeneric(".getExtendsClassSlots"),
           signature="typeMode")

setMethod(.getExtendsClassSlots, "typeMode",
    function(typeMode, rExtends, allSlots) character(0))

setMethod(.getExtendsClassSlots, "hierarchy",
    function(typeMode, rExtends, allSlots)
{
    if (length(rExtends) == 0) character(0)
    else {
        idx <- names(allSlots) %in% slotNames(rExtends)
        allSlots[idx]
    }
})

setGeneric(".getDefaultAssignStr",
           function(typeMode, slots, slotNames, rExtends)
           standardGeneric(".getDefaultAssignStr"),
           signature="typeMode")

setMethod(.getDefaultAssignStr, "typeMode",
    function(typeMode, slots, slotNames, rExtends)
{
    if (length(slotNames) == 0) character(0)
    else 
        paste("\t\tthis.", slotNames, " = ",
              getConstructorPart(slots), ";\n", sep="", collapse="")
})

setMethod(.getDefaultAssignStr, "hierarchy",
    function(typeMode, slots, slotNames, rExtends)
{
    res <- callNextMethod()
    if (length(rExtends) == 1)
        res <- paste("\t\tsuper();\n", res, sep="")
    res
})

setGeneric(".getParamAssignStr",
           function(typeMode, slotNames, extendsSlotNames, rExtends)
           standardGeneric(".getParamAssignStr"),
           signature="typeMode")

setMethod(.getParamAssignStr, "typeMode",
    function(typeMode, slotNames, extendsSlotNames, rExtends)
{
    if (length(slotNames) == 0) character(0)
    else
        paste("\t\tthis.", slotNames, " = ", slotNames, ";\n",
              sep="", collapse="")
})

setMethod(.getParamAssignStr, "hierarchy",
    function(typeMode, slotNames, extendsSlotNames, rExtends)
{
    res <- callNextMethod()
    if (length(rExtends) == 1) {
        super <- 
            paste("\t\tsuper(",
                  paste(extendsSlotNames, collapse=", "),
                  ");\n", sep="")
        res <- paste(super, res, sep="")
    }
    res
})

############################
# section 2: Creating Java Bean 
############################

setGeneric("getExtPart",
           function(typeMode, ...) standardGeneric("getExtPart"))

setMethod("getExtPart", signature(typeMode="robject"),
          function(typeMode) 
{
    paste(" extends ", primaryDataPkg(), ".RObject ",
          sep="", collapse="")
})

setMethod("getExtPart", signature(typeMode="javalib"), 
    function(typeMode, ...) " implements java.io.Serializable "
)

setMethod(getExtPart, "hierarchy",
          function(typeMode, ..., extends=extends, lookup=lookup)
{
    res <- callNextMethod()
    if (length(extends) == 1) {
        ext <- paste(lookup[[extends]]@jPackage,
                     lookup[[extends]]@jType, sep=".")
        res <- paste(" extends ", ext, " ", res, sep="", collapse="")
    }
    res
})

setGeneric("getBaseObj", function(typeMode) standardGeneric("getBaseObj"))

setMethod("getBaseObj", signature(typeMode="robject"), 
    function(typeMode) paste(primaryDataPkg(), ".RObject", sep="", collapse="")
)

setMethod("getBaseObj", signature(typeMode="javalib"), 
    function(typeMode) "Object"
)
                            
getConstructorPart <- function(jType) {
    abstractIndex <- grep("Factory$", jType)
    rvectorIndex <- grep(paste(primaryDataPkg(), "RVector", sep="."), jType, fixed=T)
    abstractIndex <- c(abstractIndex, rvectorIndex)
    arrayIndex <- grep("\\[\\]$", jType)
    res <- paste("new ", jType, "()", sep="")
    res[abstractIndex] <- "null"
    res[arrayIndex] <- paste("new ", jType[arrayIndex], "{}", sep="")
    res
}

getToStringMethod <- function(jType, slotName) { 
    arrayIndex <- grep("\\[\\]$", jType) 
    res <- paste("String.valueOf(", slotName, ")", sep=" ")  
    res[arrayIndex] <- paste("java.util.Arrays.toString(", slotName[arrayIndex], ")", sep=" ")
    res
}

getEqualsMethod <- function(jType, getterName, slotName) {
    arrayIndex <- grep("\\[\\]$", jType)
    res <- paste(getterName, ".equals(input", capName(slotName), ")", sep="")
    res[arrayIndex] <-
        paste("java.util.Arrays.equals(", getterName[arrayIndex], ", ",
              "(",jType[arrayIndex], ")input", capName(slotName[arrayIndex]), ")", sep="")
    res
}

## create the java bean corresponding to a R S4 class
## param:
##      rType: R S4 class name 
##      pkgName: java package name, ${mainDir}.${rPkgName}.data
##      slotType: a named vector where names are slot names of the R S4 class, 
##                  and values are the corresponding java data types for the slots
##                Notice that java data types are in full name (with java package 
##                  name in front)
## return: definition of the java bean in character string
createJavaBean <-
    function( javaType, pkgName, rType, slotType, classDoc,
              typeMode, lookup ) 
{
    allSlots <- slotType
    rExtends <- .getExtendsClass(rType)
    extendsSlots <- .getExtendsClassSlots(typeMode, rExtends, allSlots)
    thisSlots <- .getThisClassSlots(typeMode, rExtends, allSlots)

    beanBody <- paste( "package ", pkgName, ";\n",
                      "\n\t/**\n",
                      "\t* This file was auto-generated by R function \n",
                      "\t* createJavaBean ", date(), ". \n",
                      "\t* ", get("ClassDesc_RWebServices", classDoc), "\n",
                      "\t*/\n", 
                       sep="", collapse="")
    beanBody <- paste(beanBody, "\n\npublic class ", javaType,
                      getExtPart(typeMode, extends=rExtends, lookup=lookup),
                      " {\n", sep="", collapse="")
    if (length(allSlots) > 0) {
        ## field: rThisClassSlots
        ## constructor: slotType
        ## get/set: rThisClassSlots
        
        allSlotNames <- mangleName(names(allSlots))
        extendsSlotNames <- mangleName(names(extendsSlots))
        thisSlotNames <- mangleName(names(thisSlots))
        rSlotName <- names(thisSlots)

        fieldDef <-
            if (length(thisSlots) == 0) character(0)
            else 
                paste("\tprivate ", thisSlots, " ", thisSlotNames,
                      ";\n", sep="", collapse="")

        defaultAssignStr <-
            .getDefaultAssignStr(typeMode, thisSlots, thisSlotNames, rExtends)

        paramStr <-
            paste("\t\t", allSlots, " ", allSlotNames, sep="",
                  collapse=",\n")

        paramAssignStr <-
            .getParamAssignStr(typeMode, thisSlotNames, extendsSlotNames, rExtends)

        constructor <-
            paste("\n\tpublic ", javaType, "() {\n",
                  defaultAssignStr,
                  "\t}\n",
                  "\n\tpublic ", javaType, "(\n",
                  paramStr, "){\n",
                  paramAssignStr,
                  "\t}\n", sep="", collapse="")
        if (length(thisSlots) != 0) {
            setMethods <-
                paste("\n\t/**\n", 
                      "\t* Sets the ", thisSlotNames, " value for this ", javaType, "\n",
                      "\t*\n",
                      "\t* @param\t", thisSlotNames, "\t", get(rSlotName, classDoc), "\n",
                      "\t*/\n",
                      "\tpublic void set", capName(thisSlotNames), "(", thisSlots,
                      " ", thisSlotNames, ") {\n",
                      "\t\tthis.", thisSlotNames, " = ", thisSlotNames, ";\n",
                      "\t}\n", sep="")
            getMethods <-
                paste("\n\t/**\n", 
                      "\t* Gets the ", thisSlotNames, " value for this ", javaType, "\n",
                      "\t*\n",
                      "\t* @return\tthe value of ", thisSlotNames, "\n",
                      "\t*\t", get(rSlotName, classDoc), "\n",
                      "\t*/\n",
                      "\tpublic ", thisSlots, " get", capName(thisSlotNames), "() {\n",
                      "\t\treturn ", thisSlotNames, ";\n",
                      "\t}\n", sep="")
        } else {
            setMethods <- getMethods <- character(0)
        }
        methodPart <-
            paste(setMethods, getMethods, sep="", collapse="")
        allGetters <- paste("get", capName(allSlotNames), "()", sep="")
        printElement <-
            paste("\t\tres.append(\"", allSlotNames, "=\" + ",
                  getToStringMethod(allSlots, allGetters),
                  ");\n",
                  sep="", collapse="\t\tres.append(\", \");\n")
        equalsElementContent <-
            paste("\t\t\tObject input", capName(allSlotNames), " = obj.",
                  allGetters, ";\n",
                  "\t\t\tif((", allGetters, "==null)||(input", capName(allSlotNames),
                  "==null))\n",
                  "\t\t\t\tres = res&&(", 
                  allGetters, "==input", capName(allSlotNames), ");\n",
                  "\t\t\telse\n",
                  "\t\t\t\tres = res&&",
                  getEqualsMethod(allSlots, allGetters, allSlotNames),
                  ";\n", sep="", collapse="")
        equalsElement <-
            paste("\t\tif(res){\n",
                  "\t\t\t", javaType, " obj = (", javaType, ")inputObject;\n",
                  equalsElementContent,
                  "\t\t}\n",
                 sep="", collapse="")
    } else {
        constructor <-
            paste("\n\tpublic ", javaType, "() {}\n\n", sep="", collapse="")
        fieldDef <- methodPart <- printElement <- equalsElement <-
            character(0)
    }
    beanBody <- paste(beanBody, fieldDef, constructor, sep="", collapse="")
    printPart <-
        paste("\n\t/**\n", 
              "\t* Print this ", javaType, " to String\n",
              "\t*\n",
              "\t* @return\tcontents of ", javaType, " as a String\n",
              "\t*/\n",
              "\tpublic String toString() {\n",
              "\t\tStringBuffer res=new StringBuffer(\"{\");\n",
              printElement,
              "\t\tres.append(\"}\");\n",
              "\t\treturn res.toString();\n",
              "\t}\n", sep="", collapse="")
    equalsPart <-
        paste("\n\t/**\n", 
              "\t* Compare this ", javaType, " with the input data.\n",
              "\t*\n",
              "\t* @param inputObject\tthe data to be compared with.\n",
              "\t* @return\ttrue if the two have the same contents, false otherwise.\n",
              "\t*/\n",
              "\tpublic boolean equals(Object inputObject) {\n",
              "\t\tboolean res = getClass().equals(inputObject.getClass());\n",
              equalsElement,
              "\t\treturn res;\n",
              "\t}\n",
              sep="", collapse="")
    paste(beanBody, methodPart, printPart, equalsPart, "\n}\n",
          sep="", collapse="")
}

## create the java classes corresponding to a R ClassUnion
## param:
##      javaType: main java factory class name (short name, without package name)
##      pkgName: java package name, ${mainDir}.${rPkgName}.data
##      subklass: a named vector where names are member class names of the R ClassUnion, 
##                  and values are the corresponding java data types for them
##                Notice that java data types are in full name (with java package 
##                  name in front)
## return: a named character vector: name is java class name (short name, 
##          without package), value is the definition of the java classes 
createUnionFactory <- function( javaType, pkgName, subklass, typeMode ) {
    header <- paste(  "*\n* This file was auto-generated from R function\n",
                      "* by R function crateUnionFactory ", date(), "\n",
                      "*/\n\n", 
                      "package ", pkgName, ";\n", sep="", collapse="")
    unionFactoryBody <- paste("\n/**\n* ", javaType, ".java\n", header,
        "\n\npublic abstract class ", javaType, getExtPart(typeMode), " {\n",
        "\n\t/**\n", 
        "\t* Sets the data value for this ", javaType, "\n",
        "\t*\n",
        "\t* @param\tdata\tdata should be an instance of ", 
        paste(subklass, sep="", collapse=" or "), "\n",
        "\t*/\n", 
        "\tpublic abstract void setData( ", getBaseObj(typeMode), " data ) throws Exception;\n", 
        "\n\t/**\n", 
        "\t* Gets the data value of this ", javaType, "\n",
        "\t*\n",
        "\t* @return\tthe value of field data\n",
        "\t*/\n",
        "\tpublic abstract ", getBaseObj(typeMode), " getData();\n",
        "\n\t/**\n", 
        "\t* Compare this ", javaType, "\n",
        "\t* with the input data.\n",
        "\t*\n",
        "\t* @param inputObject\tthe data to be compared with.\n",
        "\t* @return\ttrue if the two have the same contents, false otherwise.\n",
        "\t*/\n",
        "\tpublic abstract boolean equals(Object inputObject);\n",
        "\n\t/**\n", 
        "\t* Print this ", javaType, " to String\n",
        "\t*\n",
        "\t* @return\tcontents of this ", javaType, " in String.\n",
        "\t*/\n",
        "\tpublic abstract String toString();\n",
        "}\n", sep="", collapse="") 

    factoryJType <- memberFactoryName(javaType, names(subklass))     
    memberBody <- mapply(function(jType, thisFactory) {
        defaultAssignStr <- paste("\t\tthis.data = ",
                                getConstructorPart(jType), ";\n", 
                                sep="", collapse="")
        thisBody <- paste("\n/**\n* ", thisFactory, ".java\n", header,
            "\n\npublic class ", thisFactory, " extends ", javaType, " {\n",
            "\tprivate ", jType, " data=null;\n",
            "\tprotected static final String errorMsg = ",
            "\"", thisFactory, " Field data can only be ", jType, ".\";\n",
            "\n\t/**\n", 
            "\t* Empty constructor for ", thisFactory, "\n",
            "\t*/\n",
            "\n\tpublic ", thisFactory, "() {\n",
            defaultAssignStr,
            "\t}\n",
            "\n\t/**\n", 
            "\t* Sets the data value for this ", thisFactory, "\n",
            "\t*\n",
            "\t* @param\tdata\tdata should be an instance of ", jType, "\n",
            "\t*/\n",
            "\tpublic void setData( ", getBaseObj(typeMode), " data) throws Exception {\n",
            "\t\tif( data != null ) {\n",
            "\t\t\ttry{\n",
            "\t\t\t\tthis.data = (", jType, ")data;\n",
            "\t\t\t} catch( Exception ex ) {\n",
            "\t\t\t\tthrow new Exception( ex.getMessage()+errorMsg );\n",
            "\t\t\t}\n",
            "\t\t}\n",
            "\t}\n",
            "\n\t/**\n", 
            "\t* Gets the data value of this ", thisFactory, "\n",
            "\t*\n",
            "\t* @return\tthe value of field data\n",
            "\t*/\n",
            "\tpublic ", getBaseObj(typeMode), " getData() {\n",
            "\t\t return data;\n",
            "\t}\n",
            "\n\t/**\n",
            "\n\t* Print this ", thisFactory, " to String\n",
            "\t*\n",
            "\t* @return\tcontents of this ", thisFactory, " to String\n",
            "\t*/\n",
            "\tpublic String toString() {\n",
            "\t\treturn ", getToStringMethod(jType, "data"), ";\n",
            "\t}\n",
            "\n\t/**\n", 
            "\t* Compare this ", thisFactory, "\n",
            "\t* with the input data.\n",
            "\t*\n",
            "\t* @param inputObject\tthe data to be compared with.\n",
            "\t* @return\ttrue if the two have the same contents, false otherwise.\n",
            "\t*/\n",
            "\tpublic boolean equals(Object inputObject) {\n",
            "\t\tboolean res = getClass().equals(inputObject.getClass());\n",
            "\t\tif(res){\n",
            "\t\t\t", thisFactory, " obj = (", thisFactory, ")inputObject;\n",
            "\t\t\tObject inputData = obj.getData();\n",
            "\t\t\tif((data==null)||(inputData==null))\n",
            "\t\t\t\tres=res&&(data==inputData);\n",
            "\t\t\telse\n",
            "\t\t\t\tres = res&&",
                          getEqualsMethod(jType, "data", "data"),";\n",
            "\t\t}\n",
            "\t\treturn res;\n",
            "\t}\n",
            "}\n", sep="", collapse="")
        thisBody }, subklass, factoryJType)

    factoryBody <- c(unionFactoryBody, unlist(memberBody))
    names(factoryBody) <- c(javaType, factoryJType)
    factoryBody
}

## write the java bean to a file
## param:
##      beanBody: java bean definition in character string
##      javaType: java bean name, short name (without package)
##      pkgName: java package name, ${mainDir}.${rPkgName}.data
## return: NULL
## SIDE EFFECT: create  directory ${pkgName} under current working directory
writeJavaBean <- function( beanBody, javaType, pkgName ) {
        outputPath <- gsub(".", .Platform$file.sep, pkgName, fixed=TRUE)
        dir.create(path=outputPath, showWarnings=FALSE, recursive=TRUE)
        cat(beanBody, file = 
            file.path(outputPath, paste(javaType, ".java", sep="", collapse="")))
        return()
}

## write the java classes to a file
## param:
##      beanBody: a named character vector: name is java class name (short name, 
##          without package), value is the definition of the java classes 
##      pkgName: java package name, ${mainDir}.${rPkgName}.data
## return: NULL
## SIDE EFFECT: create  directory ${pkgName} under current working directory
writeJavaFactory <- function( factoryBodies, pkgName ) {
        outputPath <- gsub(".", .Platform$file.sep, pkgName, fixed=TRUE)
        dir.create(path=outputPath, showWarnings=FALSE, recursive=TRUE)
        mapply(function(contents, name){
            cat(contents, file= 
                file.path(outputPath, paste(name, ".java", sep="", collapse="")))
            }, factoryBodies, names(factoryBodies))
        return()
}

## main function of this section
## param:
##      rType: R S4 class name 
##      jPkgName: java package name, ${mainDir}.${rPkgName}.data
##      slotListJ: a named vector where names are slot names of the R S4 class, 
##                  and values are the corresponding java data types for the slots
##                Notice that java data types are in full name (with java package 
##                  name in front)
## return: java bean name, shot name (without package name in front)
generateJavaBean <-
    function( rType, jPkgName, slotListJ, typeMode, lookup ) 
{
    javaType <- capName(mangleName(rType))
    classDoc <- getClassDoc(rType)
    beanBody <- createJavaBean(javaType, jPkgName, rType, slotListJ,
                               classDoc, typeMode, lookup)
    writeJavaBean( beanBody, javaType, jPkgName ) 
    javaType
} 

## main function of this section
## param:
##      rType: R S4 class name 
##      jPkgName: java package name, ${mainDir}.${rPkgName}.data
##      subklassJ: a named vector where names are member class names of the R ClassUnion, 
##                  and values are the corresponding java data types for them
##                Notice that java data types are in full name (with java package 
##                  name in front)
## return: java bean name, shot name (without package name in front)
generateUnionFactory <- function( rType, jPkgName, subklassJ, typeMode ) {
    javaType <- paste(capName(mangleName(rType)), "Factory", sep="", collapse="")
    factoryBodies <- createUnionFactory(javaType, jPkgName, subklassJ, typeMode)
    writeJavaFactory(factoryBodies, jPkgName)
    javaType 
}

   
