/**
 * 
 */
package org.bioconductor.packages.rservices;

/**
 * @author mtmorgan
 * 
 */
public class RJFileReferences implements java.io.Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected FileReferences fileReferences = null;

	/**
	 * 
	 */
	public RJFileReferences() {
		fileReferences = new FileReferences();
	}

	/**
	 * @param fileReferences
	 */
	public RJFileReferences(FileReferences fileReferences) {
		this.fileReferences = fileReferences;
	}

	/**
	 * @param urls
	 * @param localNames
	 * @param types
	 * @throws Exception
	 */
	public RJFileReferences(String[] urls, String[] localNames, String[] types)
			throws Exception {
		super();
		if (urls.length != localNames.length || urls.length != types.length)
			throw new Exception(
					"'urls', 'localNames', and 'types' must all have same length");
		fileReferences = new FileReferences(urls.length);
		for (int i = 0; i < urls.length; ++i) {
			FileReference ref = new FileReference(urls[i], localNames[i],
					types[i]);
			fileReferences.getFileReferences().add(ref);
		}
	}

	/**
	 * @return the urls
	 */
	public String[] getUrls() {
		String[] urls = new String[fileReferences.size()];
		for (int i = 0; i < fileReferences.size(); ++i) {
			urls[i] = fileReferences.getFileReferences().get(i).getUrl();
		}
		return urls;
	}

	/**
	 * @param urls
	 * @throws Exception
	 */
	public void setUrls(String[] urls) throws Exception {
		if (fileReferences.size() != urls.length)
			throw new Exception(
					"'urls' length does not match 'FileReferences' size");
		for (int i = 0; i < urls.length; ++i) {
			FileReference ref = fileReferences.fileReferences.get(i);
			ref.setUrl(urls[i]);
			fileReferences.getFileReferences().set(i, ref);
		}
	}

	/**
	 * @return the local names
	 */
	public String[] getLocalNames() {
		String[] localNames = new String[fileReferences.size()];
		for (int i = 0; i < fileReferences.size(); ++i) {
			localNames[i] = fileReferences.getFileReferences().get(i)
					.getLocalName();
		}
		return localNames;
	}

	/**
	 * @param localNames
	 * @throws Exception
	 */
	public void setLocalNames(String[] localNames) throws Exception {
		if (fileReferences.size() != localNames.length)
			throw new Exception(
					"'localNames' length does not match 'FileReferences' size");
		for (int i = 0; i < localNames.length; ++i) {
			FileReference ref = fileReferences.getFileReferences().get(i);
			ref.setLocalName(localNames[i]);
			fileReferences.getFileReferences().set(i, ref);
		}
	}

	/**
	 * @return the file types
	 */
	public String[] getTypes() {
		String[] types = new String[fileReferences.size()];
		for (int i = 0; i < fileReferences.size(); ++i) {
			types[i] = fileReferences.getFileReferences().get(i).getType();
		}
		return types;
	}

	/**
	 * @param types
	 * @throws Exception
	 */
	public void setTypes(String[] types) throws Exception {
		if (fileReferences.size() != types.length)
			throw new Exception(
					"'types' length does not match 'FileReferences' size");
		for (int i = 0; i < types.length; ++i) {
			FileReference ref = fileReferences.getFileReferences().get(i);
			ref.setType(types[i]);
			fileReferences.getFileReferences().set(i, ref);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result
				+ ((fileReferences == null) ? 0 : fileReferences.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final RJFileReferences other = (RJFileReferences) obj;
		if (fileReferences == null) {
			if (other.fileReferences != null)
				return false;
		} else if (!fileReferences.equals(other.fileReferences))
			return false;
		return true;
	}

}
