package org.bioconductor.packages.rservices;

import javax.jms.*;

public class RServicesConnection {
	private Connection connection;

	private String queueName;

	private long timeout;

	public RServicesConnection(Connection connection, String queueName,
			long timeout) {
		this.connection = connection;
		this.queueName = queueName;
		this.timeout = timeout;
	}

	public Object call(String funcName, Object[] data, String pkgName,
			String version) throws Exception {
		Object[] ansArray = null;
		Object ans = null;
		try {
			connection.start();
			Session session = connection.createSession(false,
					Session.AUTO_ACKNOWLEDGE);
			Destination destination = session.createQueue(queueName);
			Destination replyDestination = session.createTemporaryQueue();
			MessageProducer producer = session.createProducer(destination);
			producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
			ObjectMessage message = session.createObjectMessage(data);
			message.setJMSReplyTo(replyDestination);
			message.setStringProperty("package", pkgName);
			message.setStringProperty("version", version);
			message.setStringProperty("function", funcName);
			producer.send(message);
			MessageConsumer consumer = session.createConsumer(replyDestination);
			Message responseMsg = null;
			for (long i = 0; (i < timeout / 10) && (null == responseMsg); i++) {
				Thread.sleep(100);
				responseMsg = consumer.receive(1);
			}
			if (null == responseMsg) {
				throw new Exception("Query time out");
			}
			boolean errorMsg = responseMsg.getBooleanProperty("error");
			ansArray = (Object[]) ((ObjectMessage) responseMsg).getObject();
			ans = ansArray[0];
			session.close();
			if (errorMsg) {
				throw (Exception) ans;
			}
		} finally {
			if (connection != null) {
				connection.close();
			}
		}
		return ans;
	}

}
